#load "networkDynamicData" for dataset access
library(networkDynamicData) ##access dynamic datasets
library(statnet)
library(btergm)

data("vanDeBunt_students") ### load our dataset

vandebunt<-vanDeBunt_students
##get networks
vandebunt<-get.networks(vandebunt)

##get rid of empty networks
b<-list()
k<-1
for(i in 1:length(vandebunt)){

   if(network.edgecount(vandebunt[[i]])>0){
     b[[k]]<-vandebunt[[i]]
     k<-k+1
   }

}
vandebunt<-b


###delete negative ties
for(i in 1:length(vandebunt)){

  condition<-vandebunt[[i]]%e%"relation_type"
  condition[condition<=2 | condition>=6]<-0
  condition[condition>0]<-1
  vandebunt[[i]]%e%"friends"<-condition

  del.ids<-cbind(seq(1,length(condition),1),condition)
  del.ids<-del.ids[del.ids[,2]==0,]
  delete.edges(vandebunt[[i]],eid=del.ids[,1])

}




###############################################
##  Preparing data from adjacency matrix format
###############################################

##convert vandebunt into adjacency matrices and list of network panels
actor_atts<-list()
adj_mats<-list()

##loop over each network panel
for(i in 1:length(vandebunt)){
  adj_mats[[i]]<-as.matrix(vandebunt[[i]]) ##extract adjacency matrix
  actor_atts[[i]]<-data.frame(gender=vandebunt[[i]]%v%"gender", ##extract node attributes for each panel
                              program=vandebunt[[i]]%v%"program",
                              smoking=vandebunt[[i]]%v%"smoking")

}


###convert back into network objects
net_list<-list()

for(i in 1:length(adj_mats)){

  net_list[[i]]<-as.network(adj_mats[[i]])
  net_list[[i]]%v%"gender"<-actor_atts[[i]]$gender
  net_list[[i]]%v%"program"<-actor_atts[[i]]$program
  net_list[[i]]%v%"smoking"<-actor_atts[[i]]$smoking


}

##now we can inspect the data
net_list ##net_list is now identical to vandebunt




#############
#   TERGM
#############


###run the model
set.seed(21093) ###set seed so that results replicate


###estimate the model
TERGM_1<-mtergm(vandebunt~edges+         #equivalent to intercept
                  nodeifactor("gender")+  #receiver effect of gender
                  nodeofactor("gender")+  #sender effect of gender
                  nodematch("gender")+    #same gender
                  nodeifactor("smoking")+ #receiver effect of smoking
                  nodeofactor("smoking")+  #sender effect of smoking
                  nodematch("smoking")+    #same smoking behavior
                  nodeicov("program")+     ##receiver effect for program
                  nodeocov("program")+     ##sender effect for program
                  absdiff("program")+       ##absolute difference for program
                  delrecip+             #delayed reciprocity
                  gwesp(.7,fixed=T)+         ###geometrically weighted edgewise shared partnership
                  gwidegree(.7,fixed=T),##geometrically weighted indegree distribution
                control=control.ergm(parallel=3, #run on 3 processors to speed up estimation
                MCMC.samplesize = 3000, #increase MCMC sample size to improve convergence
                MCMLE.maxit=7)

)


summary(TERGM_1)

##examine degree distribution
degree_list<-list()
for(i in 1:length(vandebunt)){
  degree_list<-rbind(degree_list,degree(vandebunt[[i]]))

}
mean(unlist(degree_list))

###can show relation to ERGM by estimating the model via the "ergm" package in R
  ## it is often useful to estimate both models, since the "btergm" and "ergm" packages have different utilities



###prepare the TERGM data using the following function
TERGM_1_stats<-mtergm(vandebunt~edges+
                  nodeifactor("gender")+
                  nodeofactor("gender")+
                    nodematch("gender")+
                    nodeifactor("smoking")+
                    nodeofactor("smoking")+
                    nodematch("smoking")+
                    nodeicov("program")+
                    nodeocov("program")+
                    absdiff("program")+
                    delrecip+
                  gwesp(.7,fixed=T)+
                  gwidegree(.6,fixed=T),
                  returndata = T ###get the statistics for the TERGM

)



set.seed(21093) ###set seed so that results replicate

###run the model using the ERGM package
TERGM_1a<-ergm(TERGM_1_stats$networks~edges+
                 nodeifactor("gender")+
                 nodeofactor("gender")+
                 nodematch("gender")+
                 nodeifactor("smoking")+
                 nodeofactor("smoking")+
                 nodematch("smoking")+
                 nodeicov("program")+
                 nodeocov("program")+
                 absdiff("program")+
                 edgecov(TERGM_1_stats$delrecip)+
                 gwesp(.7,fixed=T)+
                 gwidegree(.6,fixed=T)+
                 offset(edgecov(TERGM_1_stats$offsmat)), ##specify lagged network
               offset.coef = -Inf,                       #specify offset parameter
               control=control.ergm(parallel=3,
                                    MCMC.samplesize = 3000,
                                    MCMLE.maxit=7))


##compare results
summary(TERGM_1)
summary(TERGM_1a)

###The only real benefit to this latter method is that there are more utilities for "ergm" objects as opposed to "mtergm" objects

##check MCMC mixing
plot(TERGM_1$sample) ##plot MCMC diagnostics

checkdegeneracy(TERGM_1,mycol="black")

##check goodness of fit
gof.TERGM_1<-gof(TERGM_1) ###can use btergm package aswell
plot(gof.TERGM_1,roc.col="black",pr.col="black",obs.col="black") ###often better to plot results

#check collinearity
TERGM_1a$covar<-TERGM_1a$covar[TERGM_1a$offset==FALSE,TERGM_1a$offset==FALSE] ##Remove offset term from covariance matrix (offset term can't covary/be collinear)
TERGM_1a$coef<-TERGM_1a$coef[TERGM_1a$offset==FALSE] #Remove offset term from coefficients (offset term can't covary/be collinear)
ergMargins::vif.ergm(TERGM_1)  ##calculate variance inflation factors


###interpretation with ergMargins
library(ergMargins)
ergm.AME(TERGM_1,var1="nodeifactor.gender.2")
ergm.AME(TERGM_1,var1="nodeofactor.gender.2")
ergm.AME(TERGM_1,var1="nodeifactor.gender.2",var2="nodeofactor.gender.2",inter="nodematch.gender")

ergm.AME(TERGM_1,var1="nodeifactor.smoking.2")
ergm.AME(TERGM_1,var1="nodeofactor.smoking.2")
ergm.AME(TERGM_1,var1="nodeifactor.smoking.2",var2="nodeofactor.smoking.2",inter="nodematch.smoking")

ergm.AME(TERGM_1,var1="nodeicov.program")
ergm.AME(TERGM_1,var1="nodeocov.program")
ergm.AME(TERGM_1,var1="nodeocov.program",var2="nodeicov.program",inter="absdiff.program")

ergm.AME(TERGM_1,var1="edgecov.delrecip")
ergm.AME(TERGM_1,var1="gwesp.fixed.0.7")
ergm.AME(TERGM_1,var1="gwidegree.fixed.0.6")




###constrain outdegree to 5
set.seed(21093)
TERGM_cd<-mtergm(vandebunt~edges+
                  nodeifactor("gender")+
                  nodeofactor("gender")+
                   nodematch("gender")+
                  nodeifactor("smoking")+
                  nodeofactor("smoking")+
                   nodematch("smoking")+
                   nodeicov("program")+
                   nodeocov("program")+
                   absdiff("program")+
                  delrecip+
                  gwesp(.7,fixed=T)+
                  gwidegree(.6,fixed=T),
                 constraints=~bd(maxout=5) ##constrain maximum outdegree to be 5

)


summary(TERGM_cd)


###use bootstrap estimation

set.seed(21093)
TERGM_bs<-btergm(vandebunt~edges+
                   nodeifactor("gender")+
                   nodeofactor("gender")+
                   nodematch("gender")+
                   nodeifactor("smoking")+
                   nodeofactor("smoking")+
                   nodematch("smoking")+
                   nodeicov("program")+
                   nodeocov("program")+
                   absdiff("program")+
                   delrecip+
                   gwesp(.7,fixed=T)+
                   gwidegree(.6,fixed=T),
                 R=1000 ##set to 1000 replications

)


summary(TERGM_bs)

##get standard errors
btergm.se(TERGM_bs)

##goodness of fit
gof.TERGM_bs<-gof(TERGM_bs) ###check fit
plot(gof.TERGM_bs)




