#load "networkDynamicData" for dataset access
library(networkDynamicData) ##access dynamic datasets
library(statnet)

data("vanDeBunt_students") ### load our dataset from networkDynamicData

vandebunt<-vanDeBunt_students
##get networks
vandebunt<-get.networks(vandebunt)

##get rid of empty networks
b<-list()
k<-1
for(i in 1:length(vandebunt)){
 
   if(network.edgecount(vandebunt[[i]])>0){
     b[[k]]<-vandebunt[[i]]
     k<-k+1
   }
  
}
vandebunt<-b


###delete negative ties
for(i in 1:length(vandebunt)){
  
  condition<-vandebunt[[i]]%e%"relation_type"
  condition[condition<=2 | condition>=6]<-0
  condition[condition>0]<-1
  vandebunt[[i]]%e%"friends"<-condition
  
  del.ids<-cbind(seq(1,length(condition),1),condition)
  del.ids<-del.ids[del.ids[,2]==0,]
  delete.edges(vandebunt[[i]],eid=del.ids[,1]) 
  
}




##################
#       SAOM
##################

###RSiena has a quirky interface, so we have to do some data preprocessing
library(RSiena)


###get adjacency matrices from networks
vandebunt_adj<-vandebunt

for(i in 1:length(vandebunt_adj)){
  
  vandebunt_adj[[i]]<-as.matrix(vandebunt[[i]])
  
}

##create "sienaDependent" object
VanDebuntNetwork<-sienaDependent(array(c(vandebunt_adj[[1]],
                                         vandebunt_adj[[2]],
                                         vandebunt_adj[[3]],
                                         vandebunt_adj[[4]],
                                         vandebunt_adj[[5]],
                                         vandebunt_adj[[6]]),
                                       dim=c(32,32,6))) ##each network is 32 x 32, observed 6 times


###Now assign variables
  ##get gender attribute
gender<-vandebunt[[1]]%v%"gender"
##gender is non-time varying, so we set the attribute using "coCovar"
Gender<-coCovar(gender)

  #get smoking behavior
smoking<-vandebunt[[1]]%v%"smoking"
Smoking<-coCovar(smoking)

  #get program length
program<-vandebunt[[1]]%v%"program"
Program<-coCovar(program)

#create the SIENA dataset
SAOM.Data<-sienaDataCreate(Network=VanDebuntNetwork,Gender,Smoking,Program)



##Now that we have the data, we need to formulate the model

  #The geteffects() function tells us what terms are in our model
SAOM.terms<-getEffects(SAOM.Data)     #Right now we only have the rate parameters, density, and reciprocity

##the effectsDocumentation command helps us narrow down what terms are available
    #effectsDocumentation(SAOM.terms)


###we'll start by including gender terms
SAOM.terms<-includeEffects(SAOM.terms,egoX,altX,sameX,interaction1="Gender") #this includes a sender effect (egoX), receiver effect (alterX), and homophily effect (sameX) for Gender
#now do smoking
SAOM.terms<-includeEffects(SAOM.terms,egoX,altX,sameX,interaction1="Smoking") #this includes a sender effect (egoX), receiver effect (alterX), and homophily effect (sameX) for Gender
#and program length
SAOM.terms<-includeEffects(SAOM.terms,egoX,altX,absDiffX,interaction1="Program") #this includes a sender effect (egoX), receiver effect (alterX), and homophily effect (sameX) for Gender

##finally, we want to include triadic effects and indegree
  ##here we'll focus on indegree popularity and transitive triplets
SAOM.terms<-includeEffects(SAOM.terms,transTies,inPop)


###Now that all of our effects are set, we create the model
create.model<-sienaAlgorithmCreate(projname="Chapter3", ##the output on model fit and convergence is stored in a "Chapter3.txt" file
                                   seed=21093, ###ensures reproducible results
                                   nsub=5,   #increased to improve convergence of siena07
                                   n3=2000) ##increased to improve convergence of siena07


##now we can estimate the model using siena07
VanDebuntModel<-siena07(create.model,
                        data=SAOM.Data,
                        effects=SAOM.terms,
                        verbose=T,  ##verbose allows us to keep an eye on estimation
                        returnDeps=T) #this returns the simulated networks used to assess convergence--we use them in the sienaGOF function to evaluate fit

VanDebuntModel


###Assess goodness of fit
indegGOF<-sienaGOF(VanDebuntModel,IndegreeDistribution,join = T,varName="Network")
outdegGOF<-sienaGOF(VanDebuntModel,OutdegreeDistribution,join = T,varName="Network")
TriadGOF<-sienaGOF(VanDebuntModel,TriadCensus,join = T,varName="Network")


plot(indegGOF)
plot(outdegGOF)
plot(TriadGOF)



###score test--test whether including gender and smoking homophily improve the model
##7, 10 are indices for each parameter
Multipar.RSiena(VanDebuntModel,7,10) # gender and smoking homophily is covariates are jointly insignificant

Multipar.RSiena(VanDebuntModel,7) ##gender homophily independently insignificant
Multipar.RSiena(VanDebuntModel,10) ##smoking homophily independently insignificant




##check collinearity
summary(VanDebuntModel) ##high correlations between indegree popularity and transitive ties--worth a re-estimation


create.model<-sienaAlgorithmCreate(projname="Chapter3",seed=21092,nsub=5,n3=2000) ###change seed
 
VanDebuntModel_collin_check<-siena07(create.model,
                        data=SAOM.Data,
                        effects=SAOM.terms,
                        verbose=T)  




#####interpret absdiff program length

altmean<-mean(program)
ego_val<-unique(program)
abs_diff<-abs(ego_val-altmean)


alt_eff<-VanDebuntModel$theta[11]*altmean
ego_eff<-VanDebuntModel$theta[12]*ego_val
abs_eff<-VanDebuntModel$theta[13]*abs_diff

prediction<-(alt_eff+ego_eff+abs_eff)

plot_data<-data.frame(prediction=prediction,
                      ego_val=ego_val)


library(ggplot2)

ggplot(plot_data,aes(x=ego_val,y=prediction))+
  geom_point()+
  theme_bw()+
  xlab("Ego program length")+
  ylab("Predicted values")+geom_line(linetype="dashed",alpha=.7,color="darkgrey")



###test absdiff parameter for significance
Multipar.RSiena(VanDebuntModel,13) ##absdiff significant at p=.007



###test for time heterogeneity
timetests<-sienaTimeTest(VanDebuntModel)
summary(timetests)



##include time dummy variables
SAOM.terms<-includeTimeDummy(SAOM.terms,density)

VanDebuntModel_timedummies<-siena07(create.model,
                        data=SAOM.Data,
                        effects=SAOM.terms,
                        verbose=T)  ##verbose allows us to keep an eye on estimation

timetests2<-sienaTimeTest(VanDebuntModel_timedummies)
summary(timetests2)

##test indegree and transitive triads
Multipar.RSiena(VanDebuntModel_timedummies,4,3, 8)
Multipar.RSiena(VanDebuntModel_timedummies,3, 8)
Multipar.RSiena(VanDebuntModel_timedummies,4, 8)
Multipar.RSiena(VanDebuntModel_timedummies,4,3)
