influenceTable.basis <- function(x, xd, netname, behname,
                                 levls=NULL, levls.alt=levls,
                                 out.ego=1, silent=FALSE, nfirst=x$nwarm+1,
                                 include.endow=FALSE, include.creation=FALSE){
  # Creates basic materials for a influence table for
  # siena data set xd, sienaFit object x,
  # network variable netname (should be a character string),
  # dependent actor variable behname (also a character string),
  # levels for ego levls, levels for alter levls.alt.
  # For effect avAlt, levls.alt refers to average values of alter's behavior;
  # for effect totAlt, levls.alt refers to total values of alter's behavior;
  # For effects avSim and totSim, levls.alt refers to
  #                    constant values of alter's behavior.
  # For effects totSim and totAlt, out.ego is the presumed outdegree of ego
  # for which the contributions to the objective function are calculated.
  #
  # This function can be used also for a sienaMeta object x.
  # Then xd must be one of the data sets used for creating x,
  # with an average ('representative') value for the mean of variable <behname>.
  #
  cat("Network",netname, "; dependent behavior",behname,".\n")
  if (include.endow && include.creation)
  {
    warning(paste('It is not meaningful to include creation',
                  ' and maintenance effects simultaneously.'))
  }
  # Obtain estimate
  if (inherits(x, "sienaBayesFit"))
  {
    require(RSienaTest)
    theta <- RSienaTest:::sienaFitThetaTable(x, fromBayes=TRUE,
                                             groupOnly=0, nfirst=nfirst)$mydf$value
    theEffects <- x$requestedEffects
    if (length(theta) != dim(theEffects)[1])
    {
      stop('mismatch between theta and effect names')
    }
    # Is this correct for all model specifications?
  }
  else
  {
    theta <- x$theta
    theEffects <- x$requestedEffects
  }
  # Obtain means
  if (inherits(xd, "sienaGroup"))
  {
    cat("A sienaGroup data object was given.\n")
    # Does the variable exist, and is it a dependent variable?
    thebeh <- xd[[1]]$depvars[[behname]]
    thenet <- xd[[1]]$depvars[[netname]]
  }
  else
  {
    thebeh <- xd$depvars[[behname]]
    thenet <- xd$depvars[[netname]]
  }
  if (is.null(thenet)){stop(paste('There is no network <',netname,'>.'))}
  if (is.null(thebeh)){
    stop(paste('There is no dependent behaviour variable <',behname,'>.'))}
  
  if (inherits(xd, "sienaGroup"))
  {
    #       zmean  <- mean(sapply(xd, function(z){mean(z$depvars[[behname]], na.rm=TRUE)}))
    ztot   <- sum(sapply(xd, function(z){sum(z$depvars[[behname]], na.rm=TRUE)}))
    ztotN  <- sum(sapply(xd, function(z){sum(!is.na(z$depvars[[behname]]))}))
    zmean  <- ztot/ztotN
    zsmean <- attr(xd, "bSim")[[behname]]
    Delta  <- attr(xd, "behRange")[,behname]
    if (is.null(levls))
    {
      levls <- Delta[1] : Delta[2]
    }
    Delta  <- Delta[2] - Delta[1]
  }
  else
  {
    zmean  <- mean(colMeans(thebeh, na.rm=TRUE))
    zsmean <- attr(thebeh, 'simMean')
    Delta  <- attr(thebeh, 'range')
    if (is.null(levls))
    {
      range2 <- attr(thebeh, 'range2')
      levls <- range2[1] : range2[2]
    }
  }
  if (is.null(levls.alt))
  {
    levls.alt <- levls
  }
  # perhaps attr(thebeh, 'simMean') fails if there are more than one dependent network?
  # is this attribute then a vector of length >= 2?
  # Note that zsmean is used only if the model includes avSim or totSim.
  replace0 <- function(k){ifelse(length(k)==0,0,k)}
  efNames <- c('linear','quad','avAlt','avSim','totAlt','totSim')
  zeff.eval <- sapply(efNames, function(s)
  {replace0(which(theEffects$name == behname &
                    (theEffects$type == 'eval') &
                    (theEffects$interaction1 %in% c(netname,'')) &
                    theEffects$include &
                    theEffects$shortName==s))})
  zeff.endow <- sapply(efNames, function(s)
  {replace0(which(theEffects$name == behname &
                    (theEffects$type == 'endow') &
                    (theEffects$interaction1 %in% c(netname,'')) &
                    theEffects$include &
                    theEffects$shortName==s))})
  zeff.creation <- sapply(efNames, function(s)
  {replace0(which(theEffects$name == behname &
                    (theEffects$type == 'creation') &
                    (theEffects$interaction1 %in% c(netname,'')) &
                    theEffects$include &
                    theEffects$shortName==s))})
  # zeff gives the indicators of the effects in x
  taketheta <- function(k){ifelse(k==0,0,theta[k])}
  ztheta <- sapply(zeff.eval,taketheta)
  if (include.endow)
  {
    ztheta <- ztheta + sapply(zeff.endow,taketheta)
  }
  if (include.creation)
  {
    ztheta <- ztheta + sapply(zeff.creation,taketheta)
  }
  if (all(ztheta == 0))
  {
    cat('All parameters found for the effect of', netname, 'on',
        behname, 'are 0.\n')
    stop('there seems to be something wrong with x, xd, netname, or behname.')
  }
  if (!silent){
    cat('Parameters found are\n')
    print(ztheta[ztheta != 0.0])
    cat('\n')
    if (include.endow)
    {
      cat('This includes evaluation and endowment (maintenance) effects.\n')
    }
    if (include.creation)
    {
      cat('This includes evaluation and creation effects.\n')
    }
  }
  found <- (ztheta != 0.0)
  if (sum(found[3:6]) >= 2){
    cat("Note: more that one influence effect was found.\n")
  }
  if (found[5] | found[6]){
    cat("Note that influence in this model depends on ego's outdegree. \n")
    cat("Outdegree of ego is given as ", out.ego, "; can be changed.\n")
  }
  if (found[4] | found[6]){
    cat("Levels of alter refer to constant values of alter's behavior.\n\n")
  } else if (found[3] | found[5]){
    cat("Levels of alter refer to average values of alter's behavior.\n\n")
  }
  flush.console()
  # ztheta contains the parameter values in x
  K <- length(levls)
  KA <- length(levls.alt)
  zalter <- rep(levls.alt,each=KA)
  zego <- rep(levls,K)
  fact <- 1:K
  alter <- factor(rep(fact,each=KA))
  coeffs <- matrix(NA, K*KA, length(efNames))
  coeffs[,1] <- (zego - zmean)
  coeffs[,2] <- (zego - zmean)*(zego - zmean)
  coeffs[,3] <- (zego - zmean)*(zalter - zmean)
  coeffs[,4] <- (1-(abs(zalter - zego)/Delta)-zsmean)
  coeffs[,5] <- out.ego*(zego - zmean)*(zalter - zmean)
  coeffs[,6] <- out.ego*(1-(abs(zalter - zego)/Delta)-zsmean)
  select <- coeffs %*% ztheta
  df <- data.frame(alter, zalter, zego, select=(coeffs%*%ztheta))
  list(df=df, zeff.eval=zeff.eval, zeff.endow=zeff.endow,
       zeff.creation=zeff.creation, ztheta=ztheta, zmean=zmean, zsmean=zsmean,
       Delta=Delta, coeffs=coeffs)
}

influenceTable <- function(x, xd, netname, behname,
                           levls, levls.alt=levls, out.ego=1, ...){
  # Creates a data frame containing the influence table for
  # siena data set xd, sienaFit object x,
  # network netname (should be a character string),
  # dependent actor variable behname (also a character string),
  # levels for ego levls, levels for alter levls.alt.
  df <- influenceTable.basis(x, xd, netname, behname,
                             levls, levls.alt, out.ego, ...)$df
  df$alter <- as.character(df$alter)
  df$zego <- as.numeric(as.character(df$zego))
  df$select <- as.numeric(as.character(df$select))
  df
}











#######################################################################
# R script SelectionTables.r                                           #
# written by Tom A.B. Snijders                                         #
# November 10, 2020                                                    #
########################################################################

########################################################################
#                                                                      #
# These are functions for constructing and presenting selection tables #
# for the interpretation of results for network dynamics               #
# obtained with the RSiena program.                                    #
# Also consult the manual, Sections 13.1 and 13.3!                     #
#                                                                      #
# The main function to use is                                          #
# selectionMatrix <- function(x,xd,name,vname,levls, levls.alt=levls)  #
# which creates a matrix containing the selection table for            #
# siena or sienaGroup data set xd,                                     #
# sienaFit, sienaBayesFit, or sienaMeta object x,                      #
# actor covariate vname (should be a character string),                #
# dependent network variable name (also a character string),           #
# levels for ego levls, levels for alter levls.alt.                    #
# Mostly levls.alt will be the same as levls,                          #
# in which case it does not have to be specified.                      #
#                                                                      #
# For models utilizing creation and/or endowment effects,              #
# by specifying include.endow=TRUE                                     #
# the sum of evaluation and endowment effects is used;                 #
# by specifying include.creation=TRUE                                  #
# the sum of evaluation and creation effects is used;                  #
# the first is relevant for maintaining existing ties,                 #
# the second for creating new ties.                                    #
#                                                                      #
# The data set is only used to get the means and similarity means      #
# which are subtracted somewhere in the effects.                       #
#                                                                      #
# It is assumed variable vname is centered;                            #
# if it is not, some things below need to be changed.                  #
#                                                                      #
# Other functions to use (see below):                                  #
# selectionTable.se (only for sienaFit objects)                        #
# Computes standard error of a linear combination of elements          #
# of the selection table.                                              #
#                                                                      #
# selectionTableWithMax                                                #
# Creates a data frame of the selection table together with            #
# the maximum per ego of the selection function over alters.           #
#                                                                      #
# selectionTable.norm  (only for sienaFit objects)                     #
# Computes the location of the social norm and its standard error      #
# for the model discussed in Snijders & Lomi (2018).                   #
#                                                                      #
# selectionTable.plot                                                  #
# Constructs a plot of the selection table using ggplot2.              #
#                                                                      #
# If these functions are used for a sienaMeta object x                 #
# (as opposed to a sienaFit object),                                   #
# xd should be one of the individual data sets used for creating x,    #
# (a single-group instead of multi-group data set)                     #
# with an average ('representative') value                             #
# for the mean of variable <name>.                                     #
#                                                                      #
# If these functions are used for a sienaBayesFit object x             #
# (as opposed to a sienaFit object),                                   #
# information about the mean and similarity mean for variable vname    #
# is taken from the first data set in the list xd.                     #
########################################################################


selectionTable.basis <- function(x, xd, name, vname,
                                 levls, levls.alt=levls, nfirst=x$nwarm+1,
                                 multiplier=1,
                                 include.endow=FALSE, include.creation=FALSE,
                                 silent=FALSE){
  # Creates basic materials for a selection table for
  # siena data set xd, sienaFit object x,
  # actor variable vname (should be a character string),
  # dependent variable name (also a character string),
  # levels for ego levls, levels for alter levls.alt.
  # The multiplier is used in case
  # the variable vname has a different natural scale,
  # and the values for ego and alter as reported should be multiplied.
  # The values given for levls and levls.alt are before this multiplication,
  # i.e., they are the values as in the data set.
  #
  # The function can be used also for a sienaMeta or sienaBayesFit object x.
  # Then xd must be one of the data sets used for creating x,
  # with an average ('representative') value for the mean of variable <vname>.
  # nfirst is used only for sienaBayesFit objects x.
  #
  dsign <- function(d){
    0.5*(ifelse(d > 0, 1, 0) + ifelse(d >= 0, 1, 0))
  }
  cat("Dependent network",name, "; actor variable",vname,".\n")
  # Obtain estimate
  if (inherits(x, "sienaBayesFit"))
  {
    require(RSienaTest)
    theta <- RSienaTest:::sienaFitThetaTable(x, fromBayes=TRUE,
                                             groupOnly=0, nfirst=nfirst)$mydf$value
    theEffects <- x$requestedEffects
    if (length(theta) != dim(theEffects)[1])
    {
      stop('mismatch between theta and effect names')
    }
    # Is this correct for all model specifications?
  }
  else
  {
    theta <- x$theta
    theEffects <- x$requestedEffects
  }
  # Obtain means
  depvar <- FALSE
  if (inherits(xd, "sienaGroup"))
  {
    cat("A sienaGroup data object was given.\n")
    # Does the variable exist, and is it a dependent variable?
    xd1 <- xd[[1]]
    thevar <- xd1$cCovars[[vname]]
    if (is.null(thevar)) {thevar <- xd1$vCovars[[vname]]}
    if (is.null(thevar))
    {
      depvar <- TRUE
      thevar <- xd1$depvars[[vname]]
    }
    if (is.null(thevar)){stop(paste('There is no actor variable <',vname,'>.'))}
    if (depvar)
    {
      #           vmean <- mean(sapply(xd, function(z){mean(z$depvars[[vname]], na.rm=TRUE)}))
      vtot <- sum(sapply(xd, function(z){sum(z$depvars[[vname]], na.rm=TRUE)}))
      vtotN <- sum(sapply(xd, function(z){sum(!is.na(z$depvars[[vname]]))}))
      vmean <- vtot/vtotN
      vsmean <- attr(xd, "bSim")[[vname]]
    }
    else
    { # constant covariates are transformed by sienaGroupCreate to varying covariates
      vmean <- attr(xd, "vCovarMean")[vname]
      vsmean <- attr(xd, "vCovarSim")[vname]
    }
  }
  else
  {
    thevar <- xd$cCovars[[vname]]
    if (is.null(thevar)) {thevar <- xd$vCovars[[vname]]}
    if (is.null(thevar))
    {
      depvar <- TRUE
      thevar <- xd$depvars[[vname]]
    }
    if (is.null(thevar)){stop(paste('There is no actor variable <',vname,'>.'))}
    if (depvar)
    {# Then the mean is not stored as an attribute
      means <- colMeans(thevar, na.rm=TRUE)
      vmean <- mean(means)
    }
    else
    {
      vmean <- attr(thevar, 'mean')
    }
    vsmean <- attr(thevar, 'simMean')
  }
  if (!depvar)
  {
    if (!attr(thevar, 'centered'))
    {
      cat('Warning: this variable is not centered. The mean is subtracted anyway; this may be wrong.\n')
    }
  }
  Delta  <- attr(thevar, 'range')
  replace0 <- function(k){ifelse(length(k)==0,0,k)}
  # If additional efNames are going to be included,
  # adapt also the coeffs matrix below and function selectionTableWithMax.
  efNames <- c('altX','altSqX','egoX','egoSqX','egoXaltX',
               'simX','diffX','diffSqX','higher','sameX','egoDiffX','egoPlusAltX')
  veff.eval <- sapply(efNames, function(s){replace0(which(theEffects$name == name &
                                                            theEffects$interaction1 == vname &
                                                            (theEffects$type == 'eval') &
                                                            theEffects$include &
                                                            theEffects$shortName==s))})
  veff.endow <- sapply(efNames, function(s){replace0(which(theEffects$name == name &
                                                             theEffects$interaction1 == vname &
                                                             (theEffects$type == 'endow') &
                                                             theEffects$include &
                                                             theEffects$shortName==s))})
  veff.creation <- sapply(efNames, function(s){replace0(which(theEffects$name == name &
                                                                theEffects$interaction1 == vname &
                                                                (theEffects$type == 'creation') &
                                                                theEffects$include &
                                                                theEffects$shortName==s))})
  
  # veff gives the indicators of the effects in x
  taketheta <- function(k){ifelse(k==0,0,theta[k])}
  vtheta <- sapply(veff.eval, taketheta)
  if (include.endow)
  {
    vtheta <- vtheta + sapply(veff.endow,taketheta)
  }
  if (include.creation)
  {
    vtheta <- vtheta + sapply(veff.creation,taketheta)
  }
  if (all(vtheta == 0))
  {
    cat('All parameters found for the effect of', vname, 'on', name, 'are 0.\n')
    stop('there seems to be something wrong with x, xd, name, or vname.')
  }
  
  if (!silent){
    cat('Parameters found are\n')
    print(vtheta[vtheta != 0.0])
    cat('Check that these parameter values are correct!\n')
    if (include.endow)
    {
      cat('This includes evaluation and endowment (maintenance) effects.\n')
    }
    if (include.creation)
    {
      cat('This includes evaluation and creation effects.\n')
    }
  }
  if (!silent)
  {
    cat('The subtracted mean value of', vname, 'is', vmean,'.\n')
    if (vtheta["simX"] != 0)
    {
      cat('The subtracted similarity mean value of', vname, 'is', vsmean,
          '.\n')
    }
  }
  flush.console()
  # vtheta contains the parameter values in x
  K <- length(levls)
  KA <- length(levls.alt)
  valter <- rep(levls.alt,K)
  vego <- rep(levls,each=KA)
  # coeffs duplicates the information in f6 in selectionTableWithMax.
  # This duplication is undesirable, but I do this anyway.
  # When maintaining this script, coeffs and f6 in selectionTable.WithMax
  # must stay in line.
  coeffs <- matrix(NA, K*KA, length(efNames))
  coeffs[,1] <- (valter - vmean)
  coeffs[,2] <- (valter - vmean)*(valter - vmean)
  coeffs[,3] <- (vego - vmean)
  coeffs[,4] <- (vego - vmean)*(vego - vmean)
  coeffs[,5] <- (vego - vmean)*(valter - vmean)
  coeffs[,6] <- (1-(abs(valter - vego)/Delta)-vsmean)
  coeffs[,7] <- (valter - vego)
  coeffs[,8] <- (valter - vego)*(valter - vego)
  coeffs[,9] <- dsign(vego - valter)
  coeffs[,10] <- 1*(vego == valter)
  coeffs[,11] <- vego*(valter - vego)
  coeffs[,12] <- (vego - vmean) + (valter - vmean)
  select <- coeffs %*% vtheta
  fact <- multiplier*(1:K)
  ego <- factor(rep(fact,each=KA), ordered=TRUE)
  vego <- multiplier*vego
  valter <- multiplier*valter
  df <- data.frame(ego,vego,valter,select)
  list(df=df, veff.eval=veff.eval, veff.endow=veff.endow,
       veff.creation=veff.creation, vtheta=vtheta, vmean=vmean, vsmean=vsmean,
       Delta=Delta, coeffs=coeffs)
}


selectionTable <- function(x, xd, name, vname,
                           levls, levls.alt=levls, nfirst=x$nwarm+1,
                           multiplier=1, ...){
  # Creates a data frame containing the selection table for
  # siena data set xd, sienaFit object x,
  # actor covariate vname (should be a character string),
  # dependent variable name (also a character string),
  # levels for ego levls, levels for alter levls.alt.
  df <- selectionTable.basis(x, xd, name, vname, levls, levls.alt, nfirst=nfirst,
                             multiplier, ...)$df
  vals <- as.character(unique(df$vego))
  df$ego <- factor(as.character(df$vego), levels=vals, ordered=TRUE)
  df$valter <- as.numeric(as.character(df$valter))
  df$select <- as.numeric(as.character(df$select))
  df
}